@@ -17,7 +17,6 @@
 from __future__ import with_statement
 
 import csv
-from datetime import datetime
 import pkg_resources
 import re
 from StringIO import StringIO
@@ -40,7 +39,7 @@ from trac.ticket.notification import TicketNotifyEmail
 from trac.timeline.api import ITimelineEventProvider
 from trac.util import as_bool, as_int, get_reporter_id, lazy
 from trac.util.datefmt import (
-    format_datetime, from_utimestamp, to_utimestamp, utc
+    datetime_now, format_datetime, from_utimestamp, to_utimestamp, utc
 )
 from trac.util.html import to_fragment
 from trac.util.text import (
@@ -49,13 +48,14 @@ from trac.util.text import (
 from trac.util.presentation import separated
 from trac.util.translation import _, tag_, tagn_, N_, ngettext
 from trac.versioncontrol.diff import get_diff_options, diff_blocks
-from trac.web import IRequestHandler, arg_list_to_args, parse_arg_list
+from trac.web.api import IRequestHandler, arg_list_to_args, parse_arg_list
 from trac.web.chrome import (
     Chrome, INavigationContributor, ITemplateProvider,
     add_ctxtnav, add_link, add_notice, add_script, add_script_data,
-    add_stylesheet, add_warning, auth_link, prevnext_nav, web_context
+    add_stylesheet, add_warning, auth_link, chrome_info_script, prevnext_nav,
+    web_context
 )
-from trac.wiki.formatter import format_to, format_to_html, format_to_oneliner
+from trac.wiki.formatter import format_to, format_to_html
 
 
 class InvalidTicket(TracError):
@@ -106,6 +106,8 @@ class TicketModule(Component):
             [TracQuery#UsingTracLinks Trac links].
             (''since 0.12'')""")
 
+    ticket_path_re = re.compile(r'/ticket/([0-9]+)$')
+
     def __init__(self):
         self._warn_for_default_attr = set()
 
@@ -147,14 +149,14 @@ class TicketModule(Component):
             return self.export_csv(req, ticket, sep='\t',
                                    mimetype='text/tab-separated-values')
         elif key == 'rss':
-            return self.export_rss(req, ticket)
+            return self._export_rss(req, ticket)
 
     # INavigationContributor methods
 
     def get_active_navigation_item(self, req):
-        if re.match(r'/newticket/?', req.path_info):
-            return 'newticket'
-        return 'tickets'
+        if self.ticket_path_re.match(req.path_info):
+            return 'tickets'
+        return 'newticket'
 
     def get_navigation_items(self, req):
         if 'TICKET_CREATE' in req.perm:
@@ -165,12 +167,12 @@ class TicketModule(Component):
     # IRequestHandler methods
 
     def match_request(self, req):
-        if req.path_info == "/newticket":
-            return True
-        match = re.match(r'/ticket/([0-9]+)$', req.path_info)
+        match = self.ticket_path_re.match(req.path_info)
         if match:
             req.args['id'] = match.group(1)
             return True
+        if req.path_info == '/newticket':
+            return True
 
     def process_request(self, req):
         if 'id' in req.args:
@@ -294,10 +296,12 @@ class TicketModule(Component):
                     SELECT t.id, tc.time, tc.author, t.type, t.summary,
                            tc.field, tc.oldvalue, tc.newvalue
                     FROM ticket_change tc
-                        INNER JOIN ticket t ON t.id = tc.ticket
-                            AND tc.time>=%s AND tc.time<=%s
-                    ORDER BY tc.time, tc.ticket
-                    """ % (ts_start, ts_stop)):
+                    INNER JOIN ticket t ON
+                        t.id = tc.ticket AND tc.time>=%%s AND tc.time<=%%s
+                    LEFT OUTER JOIN enum p ON
+                        p.type='priority' AND p.name=t.priority
+                    ORDER BY tc.time, COALESCE(p.value,'')='', %s, tc.ticket
+                    """ % db.cast('p.value', 'int'), (ts_start, ts_stop)):
                 if not (oldvalue or newvalue):
                     # ignore empty change corresponding to custom field
                     # created (None -> '') or deleted ('' -> None)
@@ -416,7 +420,6 @@ class TicketModule(Component):
     def _render_batched_timeline_event(self, context, field, event):
         tickets, verb, info, summary, status, resolution, type, \
                 description, comment, cid = event[3]
-        tickets = sorted(tickets)
         if field == 'url':
             return context.href.query(id=','.join(str(t) for t in tickets))
         elif field == 'title':
@@ -440,7 +443,7 @@ class TicketModule(Component):
                 yield controller
 
     def _process_newticket_request(self, req):
-        req.perm.require('TICKET_CREATE')
+        req.perm('ticket').require('TICKET_CREATE')
         ticket = Ticket(self.env)
 
         plain_fields = True # support for /newticket?version=0.11 GETs
@@ -503,6 +506,7 @@ class TicketModule(Component):
 
         if req.get_header('X-Requested-With') == 'XMLHttpRequest':
             data['preview_mode'] = True
+            data['chrome_info_script'] = chrome_info_script
             return 'ticket_box.html', data, None
 
         add_stylesheet(req, 'common/css/ticket.css')
@@ -521,7 +525,8 @@ class TicketModule(Component):
             escape_newlines = self.must_preserve_newlines
             rendered = format_to_html(self.env, context,
                                       req.args.get('edited_comment', ''),
-                                      escape_newlines=escape_newlines)
+                                      escape_newlines=escape_newlines) + \
+                       chrome_info_script(req)
             req.send(rendered.encode('utf-8'))
 
         req.perm('ticket', id, version).require('TICKET_VIEW')
@@ -543,10 +548,16 @@ class TicketModule(Component):
             elif action == 'diff':
                 return self._render_diff(req, ticket, data, text_fields)
         elif action == 'comment-history':
-            cnum = int(req.args['cnum'])
+            try:
+                cnum = int(req.args.get('cnum'))
+            except (TypeError, ValueError):
+                raise TracError(_("Invalid request arguments."))
             return self._render_comment_history(req, ticket, data, cnum)
         elif action == 'comment-diff':
-            cnum = int(req.args['cnum'])
+            try:
+                cnum = int(req.args.get('cnum'))
+            except (TypeError, ValueError):
+                raise TracError(_("Invalid request arguments."))
             return self._render_comment_diff(req, ticket, data, cnum)
         elif 'preview_comment' in req.args:
             field_changes = {}
@@ -643,6 +654,7 @@ class TicketModule(Component):
 
         if xhr:
             data['preview_mode'] = bool(data['change_preview']['fields'])
+            data['chrome_info_script'] = chrome_info_script
             return 'ticket_preview.html', data, None
 
         mime = Mimeview(self.env)
@@ -754,7 +766,7 @@ class TicketModule(Component):
                 remove.append(entry)
             else:
                 add.append(entry)
-        action = entry = ''
+        action = entry = None
         if remove:
             action, entry = ('remove', remove[0])
         elif add:
@@ -989,7 +1001,8 @@ class TicketModule(Component):
 
     def _get_comment_history(self, req, ticket, cnum):
         history = []
-        for version, date, author, comment in ticket.get_comment_history(cnum):
+        for version, date, author, comment in \
+                ticket.get_comment_history(cnum) or []:
             history.append({
                 'version': version, 'date': date, 'author': author,
                 'comment': _("''Initial version''") if version == 0 else '',
@@ -1140,6 +1153,13 @@ class TicketModule(Component):
         return (content.getvalue(), '%s;charset=utf-8' % mimetype)
 
     def export_rss(self, req, ticket):
+        """:deprecated: since 1.0.6, use `_export_rss` instead. Will be
+                        removed in 1.3.1.
+        """
+        content, content_type = self._export_rss(req, ticket)
+        return ''.join(content), content_type
+
+    def _export_rss(self, req, ticket):
         changes = []
         change_summary = {}
 
@@ -1177,7 +1197,8 @@ class TicketModule(Component):
         data = self._prepare_data(req, ticket, absurls=True)
         data['changes'] = changes
         output = Chrome(self.env).render_template(req, 'ticket.rss', data,
-                                                  'application/rss+xml')
+                                                  'application/rss+xml',
+                                                  iterable=True)
         return output, 'application/rss+xml'
 
     # Ticket validation and changes
@@ -1265,7 +1286,7 @@ class TicketModule(Component):
             valid = False
 
         # Validate summary length
-        if len(ticket['summary']) > self.max_summary_size:
+        if len(ticket['summary'] or '') > self.max_summary_size:
             add_warning(req, _("Ticket summary is too long (must be less "
                                "than %(num)s characters)",
                                num=self.max_summary_size))
@@ -1297,8 +1318,8 @@ class TicketModule(Component):
         ticket.insert()
 
         # Notify
+        tn = TicketNotifyEmail(self.env)
         try:
-            tn = TicketNotifyEmail(self.env)
             tn.notify(ticket, newticket=True)
         except Exception, e:
             self.log.error("Failure sending notification on creation of "
@@ -1330,14 +1351,14 @@ class TicketModule(Component):
         # -- Save changes
 
         fragment = ''
-        now = datetime.now(utc)
+        now = datetime_now(utc)
         cnum = ticket.save_changes(get_reporter_id(req, 'author'),
                                    req.args.get('comment'), when=now,
                                    replyto=req.args.get('replyto'))
         if cnum:
             fragment = '#comment:%d' % cnum
+            tn = TicketNotifyEmail(self.env)
             try:
-                tn = TicketNotifyEmail(self.env)
                 tn.notify(ticket, newticket=False, modtime=now)
             except Exception, e:
                 self.log.error("Failure sending notification on change to "
@@ -1354,7 +1375,7 @@ class TicketModule(Component):
 
         # After saving the changes, apply the side-effects.
         for controller in controllers:
-            self.log.debug("Side effect for %s" %
+            self.log.debug("Side effect for %s",
                            controller.__class__.__name__)
             controller.apply_action_side_effects(req, ticket, action)
 
@@ -1452,6 +1473,12 @@ class TicketModule(Component):
             name = field['name']
             type_ = field['type']
 
+            # ensure sane defaults
+            field.setdefault('optional', False)
+            field.setdefault('options', [])
+            field.setdefault('skip', False)
+            field.setdefault('editable', True)
+
             # enable a link to custom query for all choice fields
             if type_ not in ['text', 'textarea']:
                 field['rendered'] = self._query_link(req, name, ticket[name])
@@ -1474,6 +1501,7 @@ class TicketModule(Component):
                               for opt in field['options']]
                 milestones = [m for m in milestones
                               if 'MILESTONE_VIEW' in req.perm(m.resource)]
+                field['editable'] = milestones != []
                 groups = group_milestones(milestones, ticket.exists
                     and 'TICKET_ADMIN' in req.perm(ticket.resource))
                 field['options'] = []
@@ -1494,9 +1522,10 @@ class TicketModule(Component):
                     field['edit_label'] = {
                             'add': _("Add to Cc"),
                             'remove': _("Remove from Cc"),
-                            '': _("Add/Remove from Cc")}[cc_action]
-                    field['cc_entry'] = cc_entry or _("<Author field>")
-                    field['cc_update'] = cc_update or None
+                            None: _("Cc")}[cc_action]
+                    field['cc_action'] = cc_action
+                    field['cc_entry'] = cc_entry
+                    field['cc_update'] = cc_update
                     if cc_changed:
                         field_changes['cc']['cc_update'] = cc_update
                 if cc_changed:
@@ -1512,7 +1541,7 @@ class TicketModule(Component):
 
             # per type settings
             if type_ in ('radio', 'select'):
-                if ticket.exists:
+                if ticket.exists and field['editable']:
                     value = ticket.values.get(name)
                     options = field['options']
                     optgroups = []
@@ -1530,25 +1559,13 @@ class TicketModule(Component):
                     field['rendered'] = self._query_link(req, name, value,
                                 _("yes") if value == '1' else _("no"))
             elif type_ == 'text':
-                if field.get('format') == 'wiki':
-                    field['rendered'] = format_to_oneliner(self.env, context,
-                                                           ticket[name])
-                elif field.get('format') == 'reference':
+                if field.get('format') == 'reference':
                     field['rendered'] = self._query_link(req, name,
                                                          ticket[name])
                 elif field.get('format') == 'list':
                     field['rendered'] = self._query_link_words(context, name,
                                                                ticket[name])
-            elif type_ == 'textarea':
-                if field.get('format') == 'wiki':
-                    field['rendered'] = \
-                        format_to_html(self.env, context, ticket[name],
-                                escape_newlines=self.must_preserve_newlines)
 
-            # ensure sane defaults
-            field.setdefault('optional', False)
-            field.setdefault('options', [])
-            field.setdefault('skip', False)
             fields.append(field)
 
         # Move owner field to end when shown
