@@ -1,18 +1,31 @@
 # -*- coding: utf-8 -*-
+#
+# Copyright (C) 2006-2013 Edgewall Software
+# All rights reserved.
+#
+# This software is licensed as described in the file COPYING, which
+# you should have received as part of this distribution. The terms
+# are also available at http://trac.edgewall.org/wiki/TracLicense.
+#
+# This software consists of voluntary contributions made by many
+# individuals. For the exact contribution history, see the revision
+# history and logs, available at http://trac.edgewall.org/log/.
 
-from datetime import datetime
 import unittest
 
-from trac.util.datefmt import utc
+from trac.util.datefmt import datetime_now, utc
 from trac.wiki.model import WikiPage
 from trac.wiki.tests import formatter
 
+
 TEST_CASES = u"""
 ============================== wiki: link resolver
 wiki:TestPage
 wiki:TestPage/
 wiki:/TestPage
 [wiki:/TestPage]
+[wiki:/TestPage ]
+[wiki:/TestPage\u200B]
 [wiki:/TestPage /TestPage]
 wiki:"Space 1 23"
 wiki:"C'est l'\xe9t\xe9"
@@ -25,9 +38,11 @@ wiki:abc
 <a class="wiki" href="/wiki/TestPage">wiki:TestPage/</a>
 <a class="wiki" href="/wiki/TestPage">wiki:/TestPage</a>
 <a class="wiki" href="/wiki/TestPage">TestPage</a>
+<a class="wiki" href="/wiki/TestPage">TestPage</a>
+<a class="wiki" href="/wiki/TestPage">TestPage</a>
 <a class="wiki" href="/wiki/TestPage">/TestPage</a>
 <a class="wiki" href="/wiki/Space%201%2023">wiki:"Space 1 23"</a>
-<a class="wiki" href="/wiki/C%27est%20l%27%C3%A9t%C3%A9">wiki:"C'est l'\xe9t\xe9"</a>
+<a class="wiki" href="/wiki/C'est%20l'%C3%A9t%C3%A9">wiki:"C'est l'\xe9t\xe9"</a>
 <a class="missing wiki" href="/wiki/MissingPage" rel="nofollow">wiki:MissingPage?</a>
 <a class="missing wiki" href="/wiki/12" rel="nofollow">wiki:12?</a>
 <a class="missing wiki" href="/wiki/abc" rel="nofollow">wiki:abc?</a>
@@ -64,9 +79,13 @@ and so should <a class="missing wiki" href="/wiki/%C3%9Cberfl%C3%BCssigkeitsTheo
 ------------------------------
 ============================== More WikiPageNames conformance
 CamelCase,CamelCase.CamelCase: CamelCase
+But not CamelCase2
+nor CamelCase_
 ------------------------------
 <p>
 <a class="missing wiki" href="/wiki/CamelCase" rel="nofollow">CamelCase?</a>,<a class="missing wiki" href="/wiki/CamelCase" rel="nofollow">CamelCase?</a>.<a class="missing wiki" href="/wiki/CamelCase" rel="nofollow">CamelCase?</a>: <a class="missing wiki" href="/wiki/CamelCase" rel="nofollow">CamelCase?</a>
+But not CamelCase2
+nor CamelCase_
 </p>
 ------------------------------
 ============================== Escaping WikiPageNames
@@ -144,14 +163,14 @@ BillOfMaterials#get_bill_of_materials_from_room_xml(fpxml=nil)
 </p>
 ------------------------------
 ============================== WikiPageNames counter examples (paths)
-/absolute/path/is/NotWiki and relative/path/is/NotWiki
-/ThisIsNotWikiEither and /ThisIs/NotWikiEither but ThisIs/SubWiki
-and now This/Also.
+/absolute/path/is/NotWiki and relative/path/is/NotWiki and ../higher/is/NotWiki
+but ThisIs/SubWiki and now This/Also
+and ../Relative/Camel or /Absolute/Camel as well
 ------------------------------
 <p>
-/absolute/path/is/NotWiki and relative/path/is/NotWiki
-/ThisIsNotWikiEither and /ThisIs/NotWikiEither but <a class="missing wiki" href="/wiki/ThisIs/SubWiki" rel="nofollow">ThisIs/SubWiki?</a>
-and now <a class="missing wiki" href="/wiki/This/Also" rel="nofollow">This/Also?</a>.
+/absolute/path/is/NotWiki and relative/path/is/NotWiki and ../higher/is/NotWiki
+but <a class="missing wiki" href="/wiki/ThisIs/SubWiki" rel="nofollow">ThisIs/SubWiki?</a> and now <a class="missing wiki" href="/wiki/This/Also" rel="nofollow">This/Also?</a>
+and <a class="missing wiki" href="/wiki/Relative/Camel" rel="nofollow">../Relative/Camel?</a> or <a class="missing wiki" href="/wiki/Absolute/Camel" rel="nofollow">/Absolute/Camel?</a> as well
 </p>
 ------------------------------
 ============================== WikiPageNames counter examples (numbers)
@@ -216,10 +235,14 @@ This is a <a class="missing wiki" href="/wiki/Wiki?version=12" rel="nofollow">Wi
 ============================== WikiPageName with label
 See details of the [WikiPageNames wiki page name] syntax.
 Here's a [BadExample\fbad] example with special whitespace.
+We can also [WikiLabels '"use [quotes]"']
+or [WikiLabels "'use [quotes]'"]
 ------------------------------
 <p>
 See details of the <a class="missing wiki" href="/wiki/WikiPageNames" rel="nofollow">wiki page name?</a> syntax.
 Here's a <a class="missing wiki" href="/wiki/BadExample" rel="nofollow">bad?</a> example with special whitespace.
+We can also <a class="missing wiki" href="/wiki/WikiLabels" rel="nofollow">"use [quotes]"?</a>
+or <a class="missing wiki" href="/wiki/WikiLabels" rel="nofollow">'use [quotes]'?</a>
 </p>
 ------------------------------
 ============================== WikiPageName with label should be strict...
@@ -237,11 +260,11 @@ trac:wiki:InterTrac
 [trac:wiki:JonasBorgström jonas]
 ------------------------------
 <p>
-<a class="ext-link" href="http://trac.edgewall.org/intertrac/wiki%3AInterTrac" title="wiki:InterTrac in Trac's Trac"><span class="icon">\xa0</span>t:wiki:InterTrac</a>
-<a class="ext-link" href="http://trac.edgewall.org/intertrac/wiki%3AInterTrac" title="wiki:InterTrac in Trac's Trac"><span class="icon">\xa0</span>trac:wiki:InterTrac</a>
-<a class="ext-link" href="http://trac.edgewall.org/intertrac/wiki%3AInterTrac" title="wiki:InterTrac in Trac's Trac"><span class="icon">\xa0</span>intertrac</a>
-<a class="ext-link" href="http://trac.edgewall.org/intertrac/wiki%3AInterTrac" title="wiki:InterTrac in Trac's Trac"><span class="icon">\xa0</span>intertrac</a>
-<a class="ext-link" href="http://trac.edgewall.org/intertrac/wiki%3AJonasBorgstr%C3%B6m" title="wiki:JonasBorgström in Trac's Trac"><span class="icon">\xa0</span>jonas</a>
+<a class="ext-link" href="http://trac.edgewall.org/intertrac/wiki%3AInterTrac" title="wiki:InterTrac in Trac's Trac"><span class="icon"></span>t:wiki:InterTrac</a>
+<a class="ext-link" href="http://trac.edgewall.org/intertrac/wiki%3AInterTrac" title="wiki:InterTrac in Trac's Trac"><span class="icon"></span>trac:wiki:InterTrac</a>
+<a class="ext-link" href="http://trac.edgewall.org/intertrac/wiki%3AInterTrac" title="wiki:InterTrac in Trac's Trac"><span class="icon"></span>intertrac</a>
+<a class="ext-link" href="http://trac.edgewall.org/intertrac/wiki%3AInterTrac" title="wiki:InterTrac in Trac's Trac"><span class="icon"></span>intertrac</a>
+<a class="ext-link" href="http://trac.edgewall.org/intertrac/wiki%3AJonasBorgstr%C3%B6m" title="wiki:JonasBorgström in Trac's Trac"><span class="icon"></span>jonas</a>
 </p>
 ------------------------------
 ============================== Wiki InterTrac shorthands
@@ -252,11 +275,11 @@ trac:InterTrac
 [trac:JonasBorgström jonas]
 ------------------------------
 <p>
-<a class="ext-link" href="http://trac.edgewall.org/intertrac/InterTrac" title="InterTrac in Trac's Trac"><span class="icon">\xa0</span>t:InterTrac</a>
-<a class="ext-link" href="http://trac.edgewall.org/intertrac/InterTrac" title="InterTrac in Trac's Trac"><span class="icon">\xa0</span>trac:InterTrac</a>
-<a class="ext-link" href="http://trac.edgewall.org/intertrac/InterTrac" title="InterTrac in Trac's Trac"><span class="icon">\xa0</span>intertrac</a>
-<a class="ext-link" href="http://trac.edgewall.org/intertrac/InterTrac" title="InterTrac in Trac's Trac"><span class="icon">\xa0</span>intertrac</a>
-<a class="ext-link" href="http://trac.edgewall.org/intertrac/JonasBorgstr%C3%B6m" title="JonasBorgström in Trac's Trac"><span class="icon">\xa0</span>jonas</a>
+<a class="ext-link" href="http://trac.edgewall.org/intertrac/InterTrac" title="InterTrac in Trac's Trac"><span class="icon"></span>t:InterTrac</a>
+<a class="ext-link" href="http://trac.edgewall.org/intertrac/InterTrac" title="InterTrac in Trac's Trac"><span class="icon"></span>trac:InterTrac</a>
+<a class="ext-link" href="http://trac.edgewall.org/intertrac/InterTrac" title="InterTrac in Trac's Trac"><span class="icon"></span>intertrac</a>
+<a class="ext-link" href="http://trac.edgewall.org/intertrac/InterTrac" title="InterTrac in Trac's Trac"><span class="icon"></span>intertrac</a>
+<a class="ext-link" href="http://trac.edgewall.org/intertrac/JonasBorgstr%C3%B6m" title="JonasBorgström in Trac's Trac"><span class="icon"></span>jonas</a>
 </p>
 ------------------------------
 ============================== InterWiki links
@@ -267,18 +290,25 @@ complex link complex:a:test with positional arguments
 complex link complex:a (not enough arguments)
 complex link complex:a:test:more (too many arguments)
 
+in trac.ini inter:b:resource
+in trac.ini over:c:something overrides wiki
+
 NoLink:ignored
 NoLink:
 NoLink: ...
 ------------------------------
 <p>
-This is the original <a class="ext-link" href="http://www.usemod.com/cgi-bin/mb.pl?InterMapTxt" title="InterMapTxt in MeatBall..."><span class="icon">\xa0</span>MeatBall:InterMapTxt</a> wiki page.
-Checkout the <a class="ext-link" href="tsvn:http://svn.edgewall.com/repos/trac" title="http://svn.edgewall.com/repos/trac in tsvn"><span class="icon">\xa0</span>Trac Repository</a>.
+This is the original <a class="ext-link" href="http://www.usemod.com/cgi-bin/mb.pl?InterMapTxt" title="InterMapTxt in MeatBall..."><span class="icon"></span>MeatBall:InterMapTxt</a> wiki page.
+Checkout the <a class="ext-link" href="tsvn:http://svn.edgewall.com/repos/trac" title="http://svn.edgewall.com/repos/trac in tsvn"><span class="icon"></span>Trac Repository</a>.
 </p>
 <p>
-complex link <a class="ext-link" href="http://server/a/page/test?format=txt" title="resource test in a"><span class="icon">\xa0</span>complex:a:test</a> with positional arguments
-complex link <a class="ext-link" href="http://server/a/page/?format=txt" title="resource  in a"><span class="icon">\xa0</span>complex:a</a> (not enough arguments)
-complex link <a class="ext-link" href="http://server/a/page/test:more?format=txt" title="resource test:more in a"><span class="icon">\xa0</span>complex:a:test:more</a> (too many arguments)
+complex link <a class="ext-link" href="http://server/a/page/test?format=txt" title="resource test in a"><span class="icon"></span>complex:a:test</a> with positional arguments
+complex link <a class="ext-link" href="http://server/a/page/?format=txt" title="resource  in a"><span class="icon"></span>complex:a</a> (not enough arguments)
+complex link <a class="ext-link" href="http://server/a/page/test:more?format=txt" title="resource test:more in a"><span class="icon"></span>complex:a:test:more</a> (too many arguments)
+</p>
+<p>
+in trac.ini <a class="ext-link" href="http://inter/b/page/resource" title="Resource resource in b"><span class="icon"></span>inter:b:resource</a>
+in trac.ini <a class="ext-link" href="http://over/c/page/something" title="c:something in over"><span class="icon"></span>over:c:something</a> overrides wiki
 </p>
 <p>
 NoLink:ignored
@@ -293,11 +323,31 @@ and MeatBall:InterMapTxt?format=txt#there wiki page.
 complex link complex:a:test?go#there with positional arguments
 ------------------------------
 <p>
-See also <a class="ext-link" href="http://www.usemod.com/cgi-bin/mb.pl?InterMapTxt#there" title="InterMapTxt in MeatBall..."><span class="icon">\xa0</span>MeatBall:InterMapTxt#there</a> wiki page
-and <a class="ext-link" href="http://www.usemod.com/cgi-bin/mb.pl?InterMapTxt&amp;format=txt#there" title="InterMapTxt in MeatBall..."><span class="icon">\xa0</span>MeatBall:InterMapTxt?format=txt#there</a> wiki page.
+See also <a class="ext-link" href="http://www.usemod.com/cgi-bin/mb.pl?InterMapTxt#there" title="InterMapTxt in MeatBall..."><span class="icon"></span>MeatBall:InterMapTxt#there</a> wiki page
+and <a class="ext-link" href="http://www.usemod.com/cgi-bin/mb.pl?InterMapTxt&amp;format=txt#there" title="InterMapTxt in MeatBall..."><span class="icon"></span>MeatBall:InterMapTxt?format=txt#there</a> wiki page.
 </p>
 <p>
-complex link <a class="ext-link" href="http://server/a/page/test?format=txt&amp;go#there" title="resource test in a"><span class="icon">\xa0</span>complex:a:test?go#there</a> with positional arguments
+complex link <a class="ext-link" href="http://server/a/page/test?format=txt&amp;go#there" title="resource test in a"><span class="icon"></span>complex:a:test?go#there</a> with positional arguments
+</p>
+------------------------------
+============================== Regression for #9712
+This is not a link: x,://localhost
+------------------------------
+<p>
+This is not a link: x,:<em>localhost
+</em></p>
+------------------------------
+============================== Wiki links with @version using unicode digits
+WikiStart@₄₂
+WikiStart@₄₂#heading
+[WikiStart@₄₂]
+[WikiStart@₄₂#heading]
+------------------------------
+<p>
+<a class="wiki" href="/wiki/WikiStart">WikiStart</a>@₄₂
+<a class="wiki" href="/wiki/WikiStart">WikiStart</a>@₄₂#heading
+[<a class="wiki" href="/wiki/WikiStart">WikiStart</a>@₄₂]
+[<a class="wiki" href="/wiki/WikiStart">WikiStart</a>@₄₂#heading]
 </p>
 ------------------------------
 """ #" Emacs likes it that way better
@@ -416,6 +466,21 @@ RELATIVE_LINKS_TESTS = u"""
 <a class="missing wiki" href="/wiki/Other" rel="nofollow">Other?</a>
 </p>
 ------------------------------
+============================== Relative to the current page, as CamelCase
+OnePage/SubPage
+./SubPage
+../SiblingPage
+.././../HigherPage
+/TopPage
+------------------------------
+<p>
+<a class="missing wiki" href="/wiki/Main/OnePage/SubPage" rel="nofollow">OnePage/SubPage?</a>
+<a class="missing wiki" href="/wiki/Main/Sub/SubPage" rel="nofollow">./SubPage?</a>
+<a class="missing wiki" href="/wiki/Main/SiblingPage" rel="nofollow">../SiblingPage?</a>
+<a class="missing wiki" href="/wiki/HigherPage" rel="nofollow">.././../HigherPage?</a>
+<a class="missing wiki" href="/wiki/TopPage" rel="nofollow">/TopPage?</a>
+</p>
+------------------------------
 ============================== Relative to the current page with query strings and fragments
 [#topic see topic]
 [?param=1#topic see topic]
@@ -592,8 +657,35 @@ MissingFirstLevel/MissingPage
 """ # "
 
 
+SAFE_INTERWIKI_TESTS = u"""
+============================== InterWiki with safe_schemes
+This is the original MeatBall:InterMapTxt wiki page.
+
+Checkout the [tsvn:http://svn.edgewall.com/repos/trac Trac Repository].
+
+complex link complex:a:test with positional arguments.
+
+js:"alert(1)" javasc:"ript:alert(1)"
+------------------------------
+<p>
+This is the original <a class="ext-link" href="http://www.usemod.com/cgi-bin/mb.pl?InterMapTxt" title="InterMapTxt in MeatBall..."><span class="icon"></span>MeatBall:InterMapTxt</a> wiki page.
+</p>
+<p>
+Checkout the <a class="ext-link" href="tsvn:http://svn.edgewall.com/repos/trac" title="http://svn.edgewall.com/repos/trac in tsvn"><span class="icon"></span>Trac Repository</a>.
+</p>
+<p>
+complex link <a class="ext-link" href="http://server/a/page/test?format=txt" title="resource test in a"><span class="icon"></span>complex:a:test</a> with positional arguments.
+</p>
+<p>
+js:&#34;alert(1)&#34; javasc:&#34;ript:alert(1)&#34;
+</p>
+------------------------------
+""" # "
+
+
 def wiki_setup(tc):
-    now = datetime.now(utc)
+    tc.env.config.set('wiki', 'render_unsafe_content', True) # for #9712
+    now = datetime_now(utc)
     wiki0 = WikiPage(tc.env)
     wiki0.name = 'Main/Sub'
     wiki0.text = '--'
@@ -620,37 +712,46 @@ def wiki_setup(tc):
 This is the InterMapTxt
 ----
 {{{
-MeatBall 	http://www.usemod.com/cgi-bin/mb.pl? # $1 in MeatBall...
+MeatBall        http://www.usemod.com/cgi-bin/mb.pl? # $1 in MeatBall...
 tsvn            tsvn:
 complex         http://server/$1/page/$2?format=txt  # resource $2 in $1
+over            http://unused/? # Overridden in trac.ini
+js              javascript:
+javasc          javasc
 }}}
 ----
 {{{
 nolink          http://noweb
 }}}
-""" 
+"""
     imt.save('joe', 'test InterWiki links', '::1', now)
+    tc.env.config.set('interwiki', 'inter',
+                      'http://inter/$1/page/$2 Resource $2 in $1')
+    tc.env.config.set('interwiki', 'over',
+                      'http://over/$1/page/$2')
 
     w = WikiPage(tc.env)
     w.name = 'FirstLevel'
     w.text = '--'
     w.save('joe', 'first level of hierarchy', '::1', now)
-    
+
     w = WikiPage(tc.env)
     w.name = 'FirstLevel/SecondLevel'
     w.text = '--'
     w.save('joe', 'second level of hierarchy', '::1', now)
-    
+
     w = WikiPage(tc.env)
     w.name = 'FirstLevel/SecondLevel/ThirdLevel'
     w.text = '--'
     w.save('joe', 'third level of hierarchy', '::1', now)
-    
+
     w = WikiPage(tc.env)
     w.name = 'FirstLevel/SecondLevel/OtherThirdLevel'
     w.text = '--'
     w.save('joe', 'other third level of hierarchy', '::1', now)
 
+    tc.env.db_transaction("INSERT INTO ticket (id) VALUES ('123')")
+
 
 def wiki_teardown(tc):
     tc.env.reset_db()
@@ -661,6 +762,13 @@ def wiki_setup_split(tc):
     wiki_setup(tc)
 
 
+def wiki_setup_safe_interwiki(tc):
+    wiki_setup(tc)
+    tc.env.config.set('wiki', 'render_unsafe_content', 'false')
+    tc.env.config.set('wiki', 'safe_schemes',
+                      'file,ftp,git,irc,http,https,ssh,svn,tsvn')
+
+
 def suite():
     suite = unittest.TestSuite()
     suite.addTest(formatter.suite(TEST_CASES, wiki_setup, __file__,
@@ -673,9 +781,12 @@ def suite():
                                   context=('wiki', 'Main/Sub')))
     suite.addTest(formatter.suite(SCOPED_LINKS_TESTS, wiki_setup, __file__,
                                   wiki_teardown,
-                                  context=('wiki', 
+                                  context=('wiki',
                                       'FirstLevel/SecondLevel/ThirdLevel')))
+    suite.addTest(formatter.suite(SAFE_INTERWIKI_TESTS,
+                                  wiki_setup_safe_interwiki, __file__,
+                                  wiki_teardown))
     return suite
 
 if __name__ == '__main__':
-    unittest.main(defaultTest='suite') 
+    unittest.main(defaultTest='suite')
