@@ -16,67 +16,65 @@
 # Author: Jonas Borgström <jonas@edgewall.com>
 #         Christopher Lenz <cmlenz@gmx.de>
 
-from datetime import datetime
+from __future__ import with_statement
+
 import pkg_resources
 import re
 
-from genshi.core import Markup
 from genshi.builder import tag
 
 from trac.attachment import AttachmentModule
 from trac.config import IntOption
 from trac.core import *
-from trac.mimeview.api import Mimeview, IContentConverter, Context
+from trac.mimeview.api import IContentConverter, Mimeview
 from trac.perm import IPermissionRequestor
 from trac.resource import *
 from trac.search import ISearchSource, search_to_sql, shorten_result
 from trac.timeline.api import ITimelineEventProvider
 from trac.util import get_reporter_id
-from trac.util.datefmt import to_timestamp, utc
+from trac.util.datefmt import from_utimestamp, to_utimestamp
 from trac.util.text import shorten_line
-from trac.util.translation import _
+from trac.util.translation import _, tag_
 from trac.versioncontrol.diff import get_diff_options, diff_blocks
-from trac.web.chrome import add_link, add_script, add_stylesheet, \
-                            add_ctxtnav, add_warning, prevnext_nav, \
-                            INavigationContributor, ITemplateProvider
-from trac.web import IRequestHandler
-from trac.wiki.api import IWikiPageManipulator, WikiSystem
-from trac.wiki.formatter import format_to
+from trac.web.api import HTTPBadRequest, IRequestHandler
+from trac.web.chrome import (Chrome, INavigationContributor,
+                             ITemplateProvider, add_ctxtnav, add_link,
+                             add_notice, add_script, add_stylesheet,
+                             add_warning, prevnext_nav, web_context)
+from trac.wiki.api import IWikiPageManipulator, WikiSystem, validate_page_name
+from trac.wiki.formatter import format_to, OneLinerFormatter
 from trac.wiki.model import WikiPage
- 
+
+
 class InvalidWikiPage(TracError):
     """Exception raised when a Wiki page fails validation.
-    
+
     :deprecated: Not used anymore since 0.11
     """
- 
+
 
 class WikiModule(Component):
 
-    implements(IContentConverter, INavigationContributor, IPermissionRequestor,
-               IRequestHandler, ITimelineEventProvider, ISearchSource,
-               ITemplateProvider)
+    implements(IContentConverter, INavigationContributor,
+               IPermissionRequestor, IRequestHandler, ITimelineEventProvider,
+               ISearchSource, ITemplateProvider)
 
     page_manipulators = ExtensionPoint(IWikiPageManipulator)
 
     max_size = IntOption('wiki', 'max_size', 262144,
-        """Maximum allowed wiki page size in bytes. (''since 0.11.2'')""")
+        """Maximum allowed wiki page size in characters. (''since 0.11.2'')""")
 
     PAGE_TEMPLATES_PREFIX = 'PageTemplates/'
     DEFAULT_PAGE_TEMPLATE = 'DefaultPage'
 
     # IContentConverter methods
+
     def get_supported_conversions(self):
-        yield ('txt', _('Plain Text'), 'txt', 'text/x-trac-wiki', 'text/plain',
-               9)
+        yield ('txt', _("Plain Text"), 'txt', 'text/x-trac-wiki',
+               'text/plain', 9)
 
     def convert_content(self, req, mimetype, content, key):
-        # Tell the browser that the content should be downloaded and
-        # not rendered. The x=y part is needed to keep Safari from being 
-        # confused by the multiple content-disposition headers.
-        req.send_header('Content-Disposition', 'attachment; x=y')
-
-        return (content, 'text/plain;charset=utf-8')
+        return content, 'text/plain;charset=utf-8'
 
     # INavigationContributor methods
 
@@ -84,17 +82,19 @@ class WikiModule(Component):
         return 'wiki'
 
     def get_navigation_items(self, req):
-        if 'WIKI_VIEW' in req.perm('wiki'):
+        if 'WIKI_VIEW' in req.perm('wiki', 'WikiStart'):
             yield ('mainnav', 'wiki',
-                   tag.a(_('Wiki'), href=req.href.wiki(), accesskey=1))
+                   tag.a(_("Wiki"), href=req.href.wiki(), accesskey=1))
+        if 'WIKI_VIEW' in req.perm('wiki', 'TracGuide'):
             yield ('metanav', 'help',
-                   tag.a(_('Help/Guide'), href=req.href.wiki('TracGuide'),
+                   tag.a(_("Help/Guide"), href=req.href.wiki('TracGuide'),
                          accesskey=6))
 
     # IPermissionRequestor methods
 
     def get_permission_actions(self):
-        actions = ['WIKI_CREATE', 'WIKI_DELETE', 'WIKI_MODIFY', 'WIKI_VIEW']
+        actions = ['WIKI_CREATE', 'WIKI_DELETE', 'WIKI_MODIFY', 'WIKI_RENAME',
+                   'WIKI_VIEW']
         return actions + [('WIKI_ADMIN', actions)]
 
     # IRequestHandler methods
@@ -112,19 +112,31 @@ class WikiModule(Component):
         version = req.args.get('version')
         old_version = req.args.get('old_version')
 
-        if pagename.endswith('/'):
-            req.redirect(req.href.wiki(pagename.strip('/')))
+        if pagename.startswith('/') or pagename.endswith('/') or \
+                '//' in pagename:
+            pagename = re.sub(r'/{2,}', '/', pagename.strip('/'))
+            req.redirect(req.href.wiki(pagename))
+        if not validate_page_name(pagename):
+            raise TracError(_("Invalid Wiki page name '%(name)s'",
+                              name=pagename))
+
+        if version is not None:
+            try:
+                version = int(version)
+            except (ValueError, TypeError):
+                raise ResourceNotFound(
+                    _('No version "%(num)s" for Wiki page "%(name)s"',
+                      num=version, name=pagename))
 
         page = WikiPage(self.env, pagename)
         versioned_page = WikiPage(self.env, pagename, version=version)
 
-        req.perm(page.resource).require('WIKI_VIEW')
         req.perm(versioned_page.resource).require('WIKI_VIEW')
 
-        if version and versioned_page.version == 0 and \
-               page.version != 0:
-            raise TracError(_('No version "%(num)s" for Wiki page "%(name)s"',
-                              num=version, name=page.name))
+        if version and versioned_page.version != int(version):
+            raise ResourceNotFound(
+                _('No version "%(num)s" for Wiki page "%(name)s"',
+                  num=version, name=page.name))
 
         add_stylesheet(req, 'common/css/wiki.css')
 
@@ -132,27 +144,38 @@ class WikiModule(Component):
             if action == 'edit':
                 if 'cancel' in req.args:
                     req.redirect(req.href.wiki(page.name))
-                
+
                 has_collision = int(version) != page.version
                 for a in ('preview', 'diff', 'merge'):
                     if a in req.args:
                         action = a
                         break
+                versioned_page.text = req.args.get('text')
                 valid = self._validate(req, versioned_page)
                 if action == 'edit' and not has_collision and valid:
                     return self._do_save(req, versioned_page)
                 else:
-                    return self._render_editor(req, page, action, has_collision)
+                    return self._render_editor(req, page, action,
+                                               has_collision)
             elif action == 'delete':
                 self._do_delete(req, versioned_page)
+            elif action == 'rename':
+                return self._do_rename(req, page)
             elif action == 'diff':
-                get_diff_options(req)
+                style, options, diff_data = get_diff_options(req)
+                contextall = diff_data['options']['contextall']
                 req.redirect(req.href.wiki(versioned_page.name, action='diff',
-                                           old_version=old_version))
+                                           old_version=old_version,
+                                           version=version,
+                                           contextall=contextall or None))
+            else:
+                raise HTTPBadRequest(_("Invalid request arguments."))
         elif action == 'delete':
-            return self._render_confirm(req, versioned_page)
+            return self._render_confirm_delete(req, page)
+        elif action == 'rename':
+            return self._render_confirm_rename(req, page)
         elif action == 'edit':
-            return self._render_editor(req, versioned_page)
+            return self._render_editor(req, page)
         elif action == 'diff':
             return self._render_diff(req, versioned_page)
         elif action == 'history':
@@ -177,11 +200,11 @@ class WikiModule(Component):
 
     def _validate(self, req, page):
         valid = True
-        
+
         # Validate page size
         if len(req.args.get('text', '')) > self.max_size:
-            add_warning(req, _('The wiki page is too long (must be less '
-                               'than %(num)s characters)',
+            add_warning(req, _("The wiki page is too long (must be less "
+                               "than %(num)s characters)",
                                num=self.max_size))
             valid = False
 
@@ -190,12 +213,12 @@ class WikiModule(Component):
             for field, message in manipulator.validate_wiki_page(req, page):
                 valid = False
                 if field:
-                    add_warning(req, _("The Wiki page field '%(field)s' is "
-                                       "invalid: %(message)s",
-                                       field=field, message=message))
+                    add_warning(req, tag_("The Wiki page field '%(field)s'"
+                                          " is invalid: %(message)s",
+                                          field=field, message=message))
                 else:
-                    add_warning(req, _("Invalid Wiki page: %(message)s",
-                                       message=message))
+                    add_warning(req, tag_("Invalid Wiki page: %(message)s",
+                                          message=message))
         return valid
 
     def _page_data(self, req, page, action=''):
@@ -220,9 +243,11 @@ class WikiModule(Component):
                             ignore_space_changes='-b' in diff_options)
         def version_info(v, last=0):
             return {'path': get_resource_name(self.env, page.resource),
-                    'rev': v or 'currently edited', 
+                    # TRANSLATOR: wiki page
+                    'rev': v or _("currently edited"),
                     'shortrev': v or last + 1,
-                    'href': v and req.href.wiki(page.name, version=v) or None}
+                    'href': req.href.wiki(page.name, version=v)
+                            if v else None}
         changes = [{'diffs': diffs, 'props': [],
                     'new': version_info(new_version, old_version),
                     'old': version_info(old_version)}]
@@ -243,21 +268,79 @@ class WikiModule(Component):
         version = int(req.args.get('version', 0)) or None
         old_version = int(req.args.get('old_version', 0)) or version
 
-        db = self.env.get_db_cnx()
-        if version and old_version and version > old_version:
-            # delete from `old_version` exclusive to `version` inclusive:
-            for v in range(old_version, version):
-                page.delete(v + 1, db)
-        else:
-            # only delete that `version`, or the whole page if `None`
-            page.delete(version, db)
-        db.commit()
+        with self.env.db_transaction as db:
+            if version and old_version and version > old_version:
+                # delete from `old_version` exclusive to `version` inclusive:
+                for v in range(old_version, version):
+                    page.delete(v + 1, db)
+            else:
+                # only delete that `version`, or the whole page if `None`
+                page.delete(version, db)
 
         if not page.exists:
+            add_notice(req, _("The page %(name)s has been deleted.",
+                              name=page.name))
             req.redirect(req.href.wiki())
         else:
+            if version and old_version and version > old_version + 1:
+                add_notice(req, _("The versions %(from_)d to %(to)d of the "
+                                  "page %(name)s have been deleted.",
+                           from_=old_version + 1, to=version, name=page.name))
+            else:
+                add_notice(req, _("The version %(version)d of the page "
+                                  "%(name)s has been deleted.",
+                                  version=version, name=page.name))
             req.redirect(req.href.wiki(page.name))
 
+    def _do_rename(self, req, page):
+        if page.readonly:
+            req.perm(page.resource).require('WIKI_ADMIN')
+        else:
+            req.perm(page.resource).require('WIKI_RENAME')
+
+        if 'cancel' in req.args:
+            req.redirect(get_resource_url(self.env, page.resource, req.href))
+
+        old_name, old_version = page.name, page.version
+        new_name = req.args.get('new_name', '')
+        new_name = re.sub(r'/{2,}', '/', new_name.strip('/'))
+        redirect = req.args.get('redirect')
+
+        # verify input parameters
+        warn = None
+        if not new_name:
+            warn = _("A new name is mandatory for a rename.")
+        elif not validate_page_name(new_name):
+            warn = _("The new name is invalid (a name which is separated "
+                     "with slashes cannot be '.' or '..').")
+        elif new_name == old_name:
+            warn = _("The new name must be different from the old name.")
+        elif WikiPage(self.env, new_name).exists:
+            warn = _("The page %(name)s already exists.", name=new_name)
+        if warn:
+            add_warning(req, warn)
+            return self._render_confirm_rename(req, page, new_name)
+
+        with self.env.db_transaction as db:
+            page.rename(new_name)
+            if redirect:
+                redirection = WikiPage(self.env, old_name, db=db)
+                redirection.text = _('See [wiki:"%(name)s"].', name=new_name)
+                author = get_reporter_id(req)
+                comment = u'[wiki:"%s@%d" %s] \u2192 [wiki:"%s"].' % (
+                          new_name, old_version, old_name, new_name)
+                redirection.save(author, comment, req.remote_addr)
+
+        add_notice(req, _("The page %(old_name)s has been renamed to "
+                          "%(new_name)s.", old_name=old_name,
+                          new_name=new_name))
+        if redirect:
+            add_notice(req, _("The page %(old_name)s has been recreated "
+                              "with a redirect to %(new_name)s.",
+                              old_name=old_name, new_name=new_name))
+
+        req.redirect(req.href.wiki(old_name if redirect else new_name))
+
     def _do_save(self, req, page):
         if page.readonly:
             req.perm(page.resource).require('WIKI_ADMIN')
@@ -266,23 +349,23 @@ class WikiModule(Component):
         else:
             req.perm(page.resource).require('WIKI_MODIFY')
 
-        page.text = req.args.get('text')
         if 'WIKI_ADMIN' in req.perm(page.resource):
             # Modify the read-only flag if it has been changed and the user is
             # WIKI_ADMIN
             page.readonly = int('readonly' in req.args)
 
         try:
-            page.save(get_reporter_id(req, 'author'),
-                            req.args.get('comment'),
-                            req.remote_addr)
+            page.save(get_reporter_id(req, 'author'), req.args.get('comment'),
+                      req.remote_addr)
+            add_notice(req, _("Your changes have been saved in version "
+                              "%(version)s.", version=page.version))
             req.redirect(get_resource_url(self.env, page.resource, req.href,
                                           version=None))
         except TracError:
             add_warning(req, _("Page not modified, showing latest version."))
             return self._render_view(req, page)
 
-    def _render_confirm(self, req, page):
+    def _render_confirm_delete(self, req, page):
         if page.readonly:
             req.perm(page.resource).require('WIKI_ADMIN')
         else:
@@ -293,24 +376,46 @@ class WikiModule(Component):
             version = int(req.args.get('version', 0))
         old_version = int(req.args.get('old_version') or 0) or version
 
+        what = 'multiple' if version and old_version \
+                             and version - old_version > 1 \
+               else 'single' if version else 'page'
+
+        num_versions = 0
+        new_date = None
+        old_date = None
+        for v, t, author, comment, ipnr in page.get_history():
+            if (v <= version or what == 'page') and new_date is None:
+                new_date = t
+            if (v <= old_version and what == 'multiple' or
+                num_versions > 1 and what == 'single'):
+                break
+            num_versions += 1
+            old_date = t
+
         data = self._page_data(req, page, 'delete')
-        data.update({'new_version': None, 'old_version': None,
-                     'num_versions': 0})
+        data.update({'what': what, 'new_version': None, 'old_version': None,
+                     'num_versions': num_versions, 'new_date': new_date,
+                     'old_date': old_date})
         if version is not None:
-            num_versions = 0
-            for v,t,author,comment,ipnr in page.get_history():
-                num_versions += 1;
-                if num_versions > 1:
-                    break
-            data.update({'new_version': version, 'old_version': old_version,
-                         'num_versions': num_versions})
+            data.update({'new_version': version, 'old_version': old_version})
         self._wiki_ctxtnav(req, page)
         return 'wiki_delete.html', data, None
 
+    def _render_confirm_rename(self, req, page, new_name=None):
+        if page.readonly:
+            req.perm(page.resource).require('WIKI_ADMIN')
+        else:
+            req.perm(page.resource).require('WIKI_RENAME')
+
+        data = self._page_data(req, page, 'rename')
+        data['new_name'] = new_name if new_name is not None else page.name
+        self._wiki_ctxtnav(req, page)
+        return 'wiki_rename.html', data, None
+
     def _render_diff(self, req, page):
         if not page.exists:
-            raise TracError(_('Version %(num)s of page "%(name)s" does not '
-                              'exist',
+            raise TracError(_("Version %(num)s of page \"%(name)s\" does not "
+                              "exist",
                               num=req.args.get('version'), name=page.name))
 
         old_version = req.args.get('old_version')
@@ -329,7 +434,6 @@ class WikiModule(Component):
 
         date = author = comment = ipnr = None
         num_changes = 0
-        old_page = None
         prev_version = next_version = None
         for version, t, a, c, i in latest_page.get_history():
             if version == new_version:
@@ -342,19 +446,18 @@ class WikiModule(Component):
                     num_changes += 1
                     if not prev_version:
                         prev_version = version
-                    if (old_version and version == old_version) or \
-                            not old_version:
+                    if old_version is None or version == old_version:
                         old_version = version
-                        old_page = WikiPage(self.env, page.name, old_version)
-                        req.perm(old_page.resource).require('WIKI_VIEW')
                         break
                 else:
                     next_version = version
         if not old_version:
             old_version = 0
+        old_page = WikiPage(self.env, page.name, old_version)
+        req.perm(old_page.resource).require('WIKI_VIEW')
 
         # -- text diffs
-        old_text = old_page and old_page.text.splitlines() or []
+        old_text = old_page.text.splitlines()
         new_text = page.text.splitlines()
         diff_data, changes = self._prepare_diff(req, page, old_text, new_text,
                                                 old_version, new_version)
@@ -363,16 +466,16 @@ class WikiModule(Component):
         if prev_version:
             add_link(req, 'prev', req.href.wiki(page.name, action='diff',
                                                 version=prev_version),
-                     _('Version %(num)s', num=prev_version))
+                     _("Version %(num)s", num=prev_version))
         add_link(req, 'up', req.href.wiki(page.name, action='history'),
                  _('Page history'))
         if next_version:
             add_link(req, 'next', req.href.wiki(page.name, action='diff',
                                                 version=next_version),
-                     _('Version %(num)s', num=next_version))
+                     _("Version %(num)s", num=next_version))
 
         data = self._page_data(req, page, 'diff')
-        data.update({ 
+        data.update({
             'change': {'date': date, 'author': author, 'ipnr': ipnr,
                        'comment': comment},
             'new_version': new_version, 'old_version': old_version,
@@ -382,7 +485,8 @@ class WikiModule(Component):
             'changes': changes,
             'diff': diff_data,
         })
-        prevnext_nav(req, _('Change'), _('Wiki History'))
+        prevnext_nav(req, _("Previous Change"), _("Next Change"),
+                     _("Wiki History"))
         return 'wiki_diff.html', data, None
 
     def _render_editor(self, req, page, action='edit', has_collision=False):
@@ -395,49 +499,81 @@ class WikiModule(Component):
 
         if page.readonly:
             req.perm(page.resource).require('WIKI_ADMIN')
+        elif not page.exists:
+            req.perm(page.resource).require('WIKI_CREATE')
         else:
             req.perm(page.resource).require('WIKI_MODIFY')
         original_text = page.text
+        comment = req.args.get('comment', '')
         if 'text' in req.args:
             page.text = req.args.get('text')
         elif 'template' in req.args:
             template = self.PAGE_TEMPLATES_PREFIX + req.args.get('template')
             template_page = WikiPage(self.env, template)
             if template_page and template_page.exists and \
-                   'WIKI_VIEW' in req.perm(template_page.resource):
+                    'WIKI_VIEW' in req.perm(template_page.resource):
                 page.text = template_page.text
-        if action == 'preview':
+        elif 'version' in req.args:
+            old_page = WikiPage(self.env, page.name,
+                                version=int(req.args['version']))
+            req.perm(page.resource).require('WIKI_VIEW')
+            page.text = old_page.text
+            comment = _("Reverted to version %(version)s.",
+                        version=req.args['version'])
+        if action in ('preview', 'diff'):
             page.readonly = 'readonly' in req.args
 
         author = get_reporter_id(req, 'author')
-        comment = req.args.get('comment', '')
-        editrows = req.args.get('editrows')
-        
-        if editrows:
-            pref = req.session.get('wiki_editrows', '20')
-            if editrows != pref:
-                req.session['wiki_editrows'] = editrows
+        defaults = {'editrows': '20'}
+        prefs = dict((key, req.session.get('wiki_%s' % key, defaults.get(key)))
+                     for key in ('editrows', 'sidebyside'))
+
+        if 'from_editor' in req.args:
+            sidebyside = req.args.get('sidebyside') or None
+            if sidebyside != prefs['sidebyside']:
+                req.session.set('wiki_sidebyside', int(bool(sidebyside)), 0)
         else:
-            editrows = req.session.get('wiki_editrows', '20')
+            sidebyside = prefs['sidebyside']
+
+        if sidebyside:
+            editrows = max(int(prefs['editrows']),
+                           len(page.text.splitlines()) + 1)
+        else:
+            editrows = req.args.get('editrows')
+            if editrows:
+                if editrows != prefs['editrows']:
+                    req.session.set('wiki_editrows', editrows,
+                                    defaults['editrows'])
+            else:
+                editrows = prefs['editrows']
 
         data = self._page_data(req, page, action)
+        context = web_context(req, page.resource)
         data.update({
+            'context': context,
             'author': author,
             'comment': comment,
             'edit_rows': editrows,
+            'sidebyside': sidebyside,
             'scroll_bar_pos': req.args.get('scroll_bar_pos', ''),
             'diff': None,
+            'attachments': AttachmentModule(self.env).attachment_data(context),
         })
         if action in ('diff', 'merge'):
-            old_text = original_text and original_text.splitlines() or []
-            new_text = page.text and page.text.splitlines() or []
+            old_text = original_text.splitlines() if original_text else []
+            new_text = page.text.splitlines() if page.text else []
             diff_data, changes = self._prepare_diff(
                 req, page, old_text, new_text, page.version, '')
             data.update({'diff': diff_data, 'changes': changes,
                          'action': 'preview', 'merge': action == 'merge',
                          'longcol': 'Version', 'shortcol': 'v'})
-        
+        elif sidebyside and action != 'collision':
+            data['action'] = 'preview'
+
         self._wiki_ctxtnav(req, page)
+        Chrome(self.env).add_wiki_toolbars(req)
+        Chrome(self.env).add_auto_preview(req)
+        add_script(req, 'common/js/folding.js')
         return 'wiki_edit.html', data, None
 
     def _render_history(self, req, page):
@@ -461,7 +597,8 @@ class WikiModule(Component):
                 'ipnr': ipnr
             })
         data.update({'history': history, 'resource': page.resource})
-        add_ctxtnav(req, 'Back to '+page.name, req.href.wiki(page.name))
+        add_ctxtnav(req, _("Back to %(wikipage)s", wikipage=page.name),
+                    req.href.wiki(page.name))
         return 'history_view.html', data, None
 
     def _render_view(self, req, page):
@@ -469,13 +606,10 @@ class WikiModule(Component):
 
         # Add registered converters
         if page.exists:
-            for conversion in Mimeview(self.env).get_supported_conversions(
-                                                 'text/x-trac-wiki'):
+            for conversion in Mimeview(self.env) \
+                              .get_supported_conversions('text/x-trac-wiki'):
                 conversion_href = req.href.wiki(page.name, version=version,
                                                 format=conversion[0])
-                # or...
-                conversion_href = get_resource_url(self.env, page.resource,
-                                                req.href, format=conversion[0])
                 add_link(req, 'alternate', conversion_href, conversion[1],
                          conversion[3])
 
@@ -483,13 +617,33 @@ class WikiModule(Component):
         if page.name == 'WikiStart':
             data['title'] = ''
 
+        ws = WikiSystem(self.env)
+        context = web_context(req, page.resource)
+        higher, related = [], []
         if not page.exists:
             if 'WIKI_CREATE' not in req.perm(page.resource):
-                raise ResourceNotFound(_('Page %(name)s not found',
+                raise ResourceNotFound(_("Page %(name)s not found",
                                          name=page.name))
+            formatter = OneLinerFormatter(self.env, context)
+            if '/' in page.name:
+                parts = page.name.split('/')
+                for i in range(len(parts) - 2, -1, -1):
+                    name = '/'.join(parts[:i] + [parts[-1]])
+                    if not ws.has_page(name):
+                        higher.append(ws._format_link(formatter, 'wiki',
+                                                      '/' + name, name, False))
+            else:
+                name = page.name
+            name = name.lower()
+            related = [each for each in ws.pages
+                       if name in each.lower()
+                          and 'WIKI_VIEW' in req.perm('wiki', each)]
+            related.sort()
+            related = [ws._format_link(formatter, 'wiki', '/' + each, each,
+                                       False)
+                       for each in related]
 
         latest_page = WikiPage(self.env, page.name, version=None)
-        req.perm(latest_page.resource).require('WIKI_VIEW')
 
         prev_version = next_version = None
         if version:
@@ -506,27 +660,27 @@ class WikiModule(Component):
                             next_version = v
             except ValueError:
                 version = None
-            
+
         prefix = self.PAGE_TEMPLATES_PREFIX
-        templates = [template[len(prefix):] for template in
-                     WikiSystem(self.env).get_pages(prefix) if
-                     'WIKI_VIEW' in req.perm('wiki', template)]
+        templates = [template[len(prefix):]
+                     for template in ws.get_pages(prefix)
+                     if 'WIKI_VIEW' in req.perm('wiki', template)]
 
         # -- prev/up/next links
         if prev_version:
             add_link(req, 'prev',
                      req.href.wiki(page.name, version=prev_version),
-                     _('Version %(num)s', num=prev_version))
+                     _("Version %(num)s", num=prev_version))
 
         parent = None
         if version:
             add_link(req, 'up', req.href.wiki(page.name, version=None),
-                     _('View latest version'))
+                     _("View latest version"))
         elif '/' in page.name:
             parent = page.name[:page.name.rindex('/')]
             add_link(req, 'up', req.href.wiki(parent, version=None),
                      _("View parent page"))
-        
+
         if next_version:
             add_link(req, 'next',
                      req.href.wiki(page.name, version=next_version),
@@ -534,36 +688,40 @@ class WikiModule(Component):
 
         # Add ctxtnav entries
         if version:
-            prevnext_nav(req, _('Version'), _('View Latest Version'))
-            add_ctxtnav(req, _('Last Change'),
-                        req.href.wiki(page.name, action='diff',
-                                      version=page.version))
+            prevnext_nav(req, _("Previous Version"), _("Next Version"),
+                         _("View Latest Version"))
         else:
             if parent:
                 add_ctxtnav(req, _('Up'), req.href.wiki(parent))
             self._wiki_ctxtnav(req, page)
 
-        context = Context.from_request(req, page.resource)
+        # Plugin content validation
+        fields = {'text': page.text}
+        for manipulator in self.page_manipulators:
+            manipulator.prepare_wiki_page(req, page, fields)
+        text = fields.get('text', '')
+
         data.update({
             'context': context,
+            'text': text,
             'latest_version': latest_page.version,
             'attachments': AttachmentModule(self.env).attachment_data(context),
             'default_template': self.DEFAULT_PAGE_TEMPLATE,
             'templates': templates,
-            'version': version
+            'version': version,
+            'higher': higher, 'related': related,
+            'resourcepath_template': 'wiki_page_path.html',
         })
+        add_script(req, 'common/js/folding.js')
         return 'wiki_view.html', data, None
-    
+
     def _wiki_ctxtnav(self, req, page):
         """Add the normal wiki ctxtnav entries."""
-        add_ctxtnav(req, _('Start Page'), req.href.wiki('WikiStart'))
-        add_ctxtnav(req, _('Index'), req.href.wiki('TitleIndex'))
+        add_ctxtnav(req, _("Start Page"), req.href.wiki('WikiStart'))
+        add_ctxtnav(req, _("Index"), req.href.wiki('TitleIndex'))
         if page.exists:
-            add_ctxtnav(req, _('History'), req.href.wiki(page.name, 
+            add_ctxtnav(req, _("History"), req.href.wiki(page.name,
                                                          action='history'))
-            add_ctxtnav(req, _('Last Change'),
-                        req.href.wiki(page.name, action='diff',
-                                      version=page.version))
 
     # ITimelineEventProvider methods
 
@@ -572,23 +730,21 @@ class WikiModule(Component):
             yield ('wiki', _('Wiki changes'))
 
     def get_timeline_events(self, req, start, stop, filters):
-        db = self.env.get_db_cnx()
         if 'wiki' in filters:
             wiki_realm = Resource('wiki')
-            cursor = db.cursor()
-            cursor.execute("SELECT time,name,comment,author,version "
-                           "FROM wiki WHERE time>=%s AND time<=%s",
-                           (to_timestamp(start), to_timestamp(stop)))
-            for ts,name,comment,author,version in cursor:
+            for ts, name, comment, author, version in self.env.db_query("""
+                    SELECT time, name, comment, author, version FROM wiki
+                    WHERE time>=%s AND time<=%s
+                    """, (to_utimestamp(start), to_utimestamp(stop))):
                 wiki_page = wiki_realm(id=name, version=version)
                 if 'WIKI_VIEW' not in req.perm(wiki_page):
                     continue
-                yield ('wiki', datetime.fromtimestamp(ts, utc), author,
+                yield ('wiki', from_utimestamp(ts), author,
                        (wiki_page, comment))
 
             # Attachments
             for event in AttachmentModule(self.env).get_timeline_events(
-                req, wiki_realm, start, stop):
+                    req, wiki_realm, start, stop):
                 yield event
 
     def render_timeline_event(self, context, field, event):
@@ -596,15 +752,19 @@ class WikiModule(Component):
         if field == 'url':
             return context.href.wiki(wiki_page.id, version=wiki_page.version)
         elif field == 'title':
-            return tag(tag.em(get_resource_name(self.env, wiki_page)),
-                       wiki_page.version > 1 and ' edited' or ' created')
+            name = tag.em(get_resource_name(self.env, wiki_page))
+            if wiki_page.version > 1:
+                return tag_("%(page)s edited", page=name)
+            else:
+                return tag_("%(page)s created", page=name)
         elif field == 'description':
-            markup = format_to(self.env, None, context(resource=wiki_page),
-                               comment)
+            markup = format_to(self.env, None,
+                               context.child(resource=wiki_page), comment)
             if wiki_page.version > 1:
                 diff_href = context.href.wiki(
                     wiki_page.id, version=wiki_page.version, action='diff')
-                markup = tag(markup, ' ', tag.a('(diff)', href=diff_href))
+                markup = tag(markup,
+                             " (", tag.a(_("diff"), href=diff_href), ")")
             return markup
 
     # ISearchSource methods
@@ -616,27 +776,24 @@ class WikiModule(Component):
     def get_search_results(self, req, terms, filters):
         if not 'wiki' in filters:
             return
-        db = self.env.get_db_cnx()
-        sql_query, args = search_to_sql(db, ['w1.name', 'w1.author', 'w1.text'],
-                                        terms)
-        cursor = db.cursor()
-        cursor.execute("SELECT w1.name,w1.time,w1.author,w1.text "
-                       "FROM wiki w1,"
-                       "(SELECT name,max(version) AS ver "
-                       "FROM wiki GROUP BY name) w2 "
-                       "WHERE w1.version = w2.ver AND w1.name = w2.name "
-                       "AND " + sql_query, args)
+        with self.env.db_query as db:
+            sql_query, args = search_to_sql(db, ['w1.name', 'w1.author',
+                                                 'w1.text'], terms)
+            wiki_realm = Resource('wiki')
+            for name, ts, author, text in db("""
+                    SELECT w1.name, w1.time, w1.author, w1.text
+                    FROM wiki w1,(SELECT name, max(version) AS ver
+                                  FROM wiki GROUP BY name) w2
+                    WHERE w1.version = w2.ver AND w1.name = w2.name
+                    AND """ + sql_query, args):
+                page = wiki_realm(id=name)
+                if 'WIKI_VIEW' in req.perm(page):
+                    yield (get_resource_url(self.env, page, req.href),
+                           '%s: %s' % (name, shorten_line(text)),
+                           from_utimestamp(ts), author,
+                           shorten_result(text, terms))
 
-        wiki_realm = Resource('wiki')
-        for name, ts, author, text in cursor:
-            page = wiki_realm(id=name)
-            if 'WIKI_VIEW' in req.perm(page):
-                yield (get_resource_url(self.env, page, req.href),
-                       '%s: %s' % (name, shorten_line(text)),
-                       datetime.fromtimestamp(ts, utc), author,
-                       shorten_result(text, terms))
-        
         # Attachments
         for result in AttachmentModule(self.env).get_search_results(
-            req, wiki_realm, terms):
+                req, wiki_realm, terms):
             yield result
